-- ClickDown by Andromerius (asilva) v3.0

-- Database defaults and quick reference

local ClickDown = LibStub("AceAddon-3.0"):NewAddon("ClickDown", "AceConsole-3.0", "AceEvent-3.0")

local globalDB
local options = {
    name = "ClickDown",
    handler = ClickDown,
    type = "group",
    args = {
        workInCombat = {
            order = 1,
            type = "toggle",
            name = "Mouse button down IN combat.",
            desc = "Activates mouse button down clicks in combat.",
            width = "full",
            get = "IsworkInCombat",
            set = "ToggleworkInCombat",
        },
        workOutofCombat = {
            order = 2,
            type = "toggle",
            name = "Mouse button down OUT of combat.(NEED /reload)",
            desc = "Also activates mouse button down clicks out of combat.",
            width = "full",
            get = "IsworkOutofCombat",
            set = "ToggleworkOutofCombat",
        },
    },
}

local defaults = {
    profile = {
        workInCombat = true,
        workOutofCombat = false,
    },
}

local buttons = {
    "ActionButton",
    "MultiBarBottomLeftButton",
    "MultiBarBottomRightButton",
    "MultiBarRightButton",
    "MultiBarLeftButton",
    "PetActionButton",
    "ShapeshiftButton",
}

-- Initialize addon

function ClickDown:OnInitialize()
    -- Register the Database
    self.db = LibStub("AceDB-3.0"):New("ClickDownDB", defaults, true)
    globalDB = self.db.global

    LibStub("AceConfig-3.0"):RegisterOptionsTable("ClickDown", options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ClickDown", "ClickDown")
    self:RegisterChatCommand("clickdown", "ChatCommand")
    self:RegisterChatCommand("ClickDown", "ChatCommand")

    -- Initialize combat checks and set addon status
    self:SetupClickMode()
end

function ClickDown:SetupClickMode()
    local function SetClickMode(mode)
        for _, buttonName in ipairs(buttons) do
            for i = 1, 12 do
                local button = _G[buttonName .. i]
                if button then
                    button:RegisterForClicks(mode)
                end
            end
        end
    end

    local function UpdateClickMode()
        local inCombat = UnitAffectingCombat("player")
        local workInCombat = self.db.profile.workInCombat
        local workOutofCombat = self.db.profile.workOutofCombat

        if inCombat and workInCombat then
            SetClickMode("AnyDown")
        elseif inCombat and not workInCombat then
            SetClickMode("AnyUp")
        elseif not inCombat and workOutofCombat then
            SetClickMode("AnyDown")
        else
            SetClickMode("AnyUp")
        end
    end

    self:RegisterEvent("PLAYER_ENTERING_WORLD", UpdateClickMode)
    self:RegisterEvent("PLAYER_REGEN_DISABLED", UpdateClickMode)
    self:RegisterEvent("PLAYER_REGEN_ENABLED", UpdateClickMode)
end

-- Command line options
function ClickDown:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0"):HandleCommand("clickdown", "ClickDown", input)
    end
end

-- Options db functions
function ClickDown:IsworkInCombat(info)
    return self.db.profile.workInCombat
end

function ClickDown:ToggleworkInCombat(info, value)
    self.db.profile.workInCombat = value
end

function ClickDown:IsworkOutofCombat(info)
    return self.db.profile.workOutofCombat
end

function ClickDown:ToggleworkOutofCombat(info, value)
    self.db.profile.workOutofCombat = value
end
